unit CAModule;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ASN1, MPX509, MPDL;

type
  TdmCA = class(TDataModule)
    CACert: TX509Certificate;
    X509TrustedCertificates1: TX509TrustedCertificates;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    FCAPriv: TDLPrivateKey;
    FCAPubl: TDLPublicKey;
  public                     
    function HandleCertRequest(const CertReq: TASN1Struct;
                               OutputStream: TStream): Boolean;
    function RevokeCertificate(const SerialNumber: string): Boolean;
    property RootPublicKey: TDLPublicKey read FCAPubl;
  end;

//var
//  dmCA: TdmCA;

implementation

uses
  PKIX, X509, PKCS10, CA_CRLDemoMain, RootCAModule;

{$R *.DFM}

var
  dmCACounter: Integer = 0;

procedure TdmCA.DataModuleCreate(Sender: TObject);
var
  PK: TX509PublicKey;
  CR: TASN1Struct;
  SS: TStringStream;
  DP: TX509CRLDistPoints;
  Status: TCertStatusCode;
begin
  Inc(dmCACounter);

  X509TrustedCertificates1.AddCertificate(dmRootCA.RootCACert.Cert,True,Status);

  DLKeys(dmRootCA.RootPublicKey.Params,FCAPriv.X,FCAPubl.Y);
  CopyDLSystemParams(FCAPriv.Params,dmRootCA.RootPublicKey.Params);
  CopyDLSystemParams(FCAPubl.Params,dmRootCA.RootPublicKey.Params);
  PK := TX509PublicKey.Create;
  try
    TranslateDSAPublicKey(FCAPubl,PK);
    CACert.ImposePublicKey(PK);
  finally
    PK.Free;
  end;

  CACert.Subject.commonName := 'CA' + IntToStr(dmCACounter);

  CACert.KeyUsage := [cRLSign,keyCertSign];
  CR := nil;
  try
    SetLength(DP,1);
    DP[0].distributionPoint.Tag := 0;
    SetLength(DP[0].distributionPoint.FullName,1);
    DP[0].distributionPoint.FullName[0].Tag := 1;
    DP[0].distributionPoint.FullName[0].rfc822Name := 'http://www.acme.com/revoked.crl';
    DP[0].reasons := [rfKeyCompromise];
    CACert.ComposeCACertRequest(CR,'',DP,0);
    DSASignCertReq(CR,FCAPriv);
    SS := TStringStream.Create('');
    try
      if not dmRootCA.HandleCertRequest(CR,SS) then
        raise Exception.Create('Root CA did not accept Certificate Request');
      SS.Position := 0;
      CACert.LoadFromStream(SS,fmtDER);
      X509TrustedCertificates1.SignatureCert := CACert.Cert;
      X509TrustedCertificates1.AddCertificate(CACert.Cert,False,Status);
    finally
      SS.Free;
    end;
  finally
    CR.Free;
  end;
end;

procedure TdmCA.DataModuleDestroy(Sender: TObject);
begin
  DisposeDLPrivateKey(FCAPriv);
  DisposeDLPublicKey(FCAPubl);
end;

function TdmCA.HandleCertRequest(const CertReq: TASN1Struct;
  OutputStream: TStream): Boolean;
var
  Cert: TASN1Struct;
  Validity: TX509Validity;
  Status: TCertStatusCode;
begin
  Result := CheckSignatureCR(CertReq);
  if Result then begin
    Cert := nil;
    try
      Validity.notBefore := Now;
      Validity.notAfter := Now + 365;
      ComposeCertificateCR(Cert,CertReq,CACert.Cert,Validity);
      DSASignCertificate(Cert,CACert.Cert,FCAPriv);
      X509TrustedCertificates1.AddCertificate(Cert,False,Status);
      Cert.SaveToStream(OutputStream,fmtDER);
    finally
      Cert.Free;
    end;
  end;
end;

function TdmCA.RevokeCertificate(const SerialNumber: string): Boolean;
var
  Cert, CRL: PASN1Struct;
  IDPs: TX509IssuingDistributionPoints;
  SS: TStringStream;
begin
  Result := X509TrustedCertificates1.FindCert(SerialNumber,Cert);
  if Result then begin
    Result := X509TrustedCertificates1.RevokeCertificate(Cert^,
                                                         keyCompromise,
                                                         '',
                                                         Now,
                                                         Now+1,
                                                         FCAPriv);
    if Result then begin
      SuggestIssuingDistributionPoints(CACert.Cert,[rfKeyCompromise],IDPs);
      if X509TrustedCertificates1.FindCRL(CACert.Cert,IDPs[0],CRL) then begin
        SS := TStringStream.Create('');
        try
          CRL.SaveToStream(SS,fmtDER);
          SS.Position := 0;
          dmRootCA.AddCRL(SS);
        finally
          SS.Free;
        end;
      end else
        raise Exception.Create('CRL not found');
    end;
  end;
end;

end.
