unit CA_CRLDemoMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MPDL, StdCtrls, CAModule, ComCtrls;

type
  TfrmCA_CRLDemoMain = class(TForm)
    GroupBox1: TGroupBox;
    btnRootCA: TButton;
    btnCA1: TButton;
    btnCA2: TButton;
    GroupBox2: TGroupBox;
    ListView1: TListView;
    btnAddUser: TButton;
    btnRevoke: TButton;
    btnVerify: TButton;
    procedure btnRootCAClick(Sender: TObject);
    procedure btnCA1Click(Sender: TObject);
    procedure btnCA2Click(Sender: TObject);
    procedure ListView1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure btnAddUserClick(Sender: TObject);
    procedure btnVerifyClick(Sender: TObject);
    procedure btnRevokeClick(Sender: TObject);
  private
    FCA1: TdmCA;
    FCA2: TdmCA;
  public
  end;

var
  frmCA_CRLDemoMain: TfrmCA_CRLDemoMain;

implementation

uses
  ASN1, PKIX, X509, RootCAModule, PKCS10, SecUtils, MPX509;

{$R *.DFM}

procedure TfrmCA_CRLDemoMain.btnRootCAClick(Sender: TObject);
begin
  dmRootCA.CreateKeyAndCert;
  btnRootCA.Enabled := False;
  btnCA1.Enabled := True;
  btnCA2.Enabled := True;
end;

procedure TfrmCA_CRLDemoMain.btnCA1Click(Sender: TObject);
begin
  FCA1 := TdmCA.Create(Application);
  btnCA1.Enabled := False;
  btnAddUser.Enabled := True;
end;

procedure TfrmCA_CRLDemoMain.btnCA2Click(Sender: TObject);
begin
  FCA2 := TdmCA.Create(Application);
  btnCA2.Enabled := False;   
  btnAddUser.Enabled := True;
end;

procedure TfrmCA_CRLDemoMain.ListView1SelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  btnRevoke.Enabled := Assigned(Item) and Selected;
  btnVerify.Enabled := btnRevoke.Enabled;
end;

procedure TfrmCA_CRLDemoMain.FormDestroy(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to ListView1.Items.Count - 1 do
    TObject(ListView1.Items[I].Data).Free;
end;

procedure TfrmCA_CRLDemoMain.btnAddUserClick(Sender: TObject);
var
  UserName, PW, Digest, Serial: string;
  Hash: TSHA1;
  UName: TX501Name;
  Priv: TDLPrivateKey;
  Publ: TDLPublicKey;
  PK: TX509PublicKey;
  CertReq, Cert: TASN1Struct;
  SS: TStringStream;
  LI: TListItem;
begin
  if not InputQuery('Add user','Enter username',UserName) then Exit;

  UName.commonName.Str := UserName;

  FillChar(Priv,SizeOf(Priv),0);
  FillChar(Publ,SizeOf(Publ),0);
  try
    DLKeys(dmRootCA.RootPublicKey.Params,Priv.X,Publ.Y);
    CopyDLSystemParams(Priv.Params,dmRootCA.RootPublicKey.Params);
    CopyDLSystemParams(Publ.Params,dmRootCA.RootPublicKey.Params);

    CertReq := nil;
    try
      PK := nil;
      try
        TranslateDHPublicKey(Publ,PK);
        PW := 'abc' + UserName;
        Hash := TSHA1.Create(PW[1],Length(PW));
        try 
          SetLength(Digest,20);
          Hash.Done(Pointer(Digest));
        finally
          Hash.Free;
        end;
        ComposeCertRequest(CertReq,UName,PK,Digest,[encipherOnly,keyAgreement],nil,nil);
        DSASignCertReq(CertReq,Priv);
        SS := TStringStream.Create('');
        try
          if Assigned(FCA2) and Assigned(FCA1) then begin
            if Random(1) = 0 then
              FCA1.HandleCertRequest(CertReq,SS)
            else
              FCA2.HandleCertRequest(CertReq,SS);
          end else if Assigned(FCA2) then     
            FCA2.HandleCertRequest(CertReq,SS)
          else
            FCA1.HandleCertRequest(CertReq,SS);
          SS.Position := 0;
          Cert := nil;
          ASN1FromStream(SS,Cert);

          LI := ListView1.Items.Add;
          LI.Data := Cert;
          LI.Caption := UserName;
          ExtractSerial(Cert,Serial);
          LI.SubItems.Add(OSToHex(Serial));
          LI.SubItems.Add('untested');
        finally
          SS.Free;
        end;
      finally
        PK.Free;
      end;
    finally
      CertReq.Free;
    end;
  finally
    DisposeDLPrivateKey(Priv);
    DisposeDLPublicKey(Publ);
  end;
end;

procedure TfrmCA_CRLDemoMain.btnVerifyClick(Sender: TObject);
var
  LI: TListItem;
  Cert: TASN1Struct;
  SS: TStringStream;
  Status: TCertStatusCode;
  SC: string;
begin
  LI := ListView1.Selected;
  Cert := LI.Data;
  SS := TStringStream.Create('');
  try
    Cert.SaveToStream(SS,fmtDER);
    SS.Position := 0;
    dmRootCA.VerifyCertificate(SS,Status);
    case Status of
      crcOK:                   SC := 'OK';
      crcExpired:              SC := 'Expired';
      crcInvalidSignature:     SC := 'Invalid';
      crcCANotTrusted:         SC := 'CA not trusted';
      crcRevoked:              SC := 'Revoked';
      crcCAExpiredOrRevoked:   SC := 'Invalid CA cert';
      crcConstraintFault:      SC := 'Constraint fault';
      crcInvalidKeyUsage:      SC := 'Invalid key usage';
      crcUnsupportedExtension: SC := 'Unsupported extension';
      crcSuperseeded:          SC := 'Superseeded';
      crcSyntax:               SC := 'Syntax error';
      crcBaseNotFound:         SC := 'Base not found';
      crcTrusted:              SC := 'Trusted';
    end;
    LI.SubItems[1] := SC;
  finally
    SS.Free;
  end;
end;

procedure TfrmCA_CRLDemoMain.btnRevokeClick(Sender: TObject);
var
  LI: TListItem;
  SN: string;
begin
  LI := ListView1.Selected;
  SN := HexToOS(LI.SubItems[0]);
  if Assigned(FCA1) then
    FCA1.RevokeCertificate(SN);
  if Assigned(FCA2) then
    FCA2.RevokeCertificate(SN);
  btnVerifyClick(Sender);
end;

end.
